proc a_tower
rem
rem   tower_prg                                       B Cable   1/96
rem
rem    tower_prg is an ARCHIVE program that displays and solves the Tower
rem    of Hanoi puzzle for the specified number of disks. Originally appearing
rem    in UPDATE magazine but released to the public domain.
rem
endproc 
proc close_pegs
rem
rem   close_pegs
rem
rem    close_pegs will close peg file logical name "p" if it is open
rem    If it is not open an error will occur which should be trapped by
rem    the calling proceudre
rem
close "p"
endproc 
proc create_pegs;ndisks
rem
rem   create_pegs
rem
rem    create_pegs closes and deletes pegs_dbf if it exists then creates a
rem    a new set of pegs with one record for each disk to be used. The peg_dbf
rem    database is expected to be on device td$(target device) which should be
rem    set to ram1_ in the tower procedure. The passed argument ndisks is the
rem    number of disks to have on the first peg.
rem
error close_pegs
kill td$+"peg_dbf"
create td$+"peg_dbf" logical "p"
peg1
peg2
peg3
endcreate 
let i=0: while i<ndisks
let peg1=ndisks-i: append : let i=i+1: endwhile 
endproc 
proc disk_show;peg,disk,level
rem
rem   disk_show
rem
rem    disk_show will display the specified disk on the specified peg.
rem    Each disk is associated with a letter 1 - A, 2 - B, etc which is
rem    is used in the display. A larger disk number means a larger disk.
rem    The level tells the procedure where it should go on the peg. Each peg
rem    has a different position on the screen (1 = 15, 2 = 40, 3 = 65)
rem    The chr(64+disk) changes the disk number into a display letter.
rem 
if peg=1: let peg=15
else : if peg=2: let peg=40
else : let peg=65: endif : endif 
print at 20-level,peg-disk;chr(64+disk);rept(chr(64+disk),2*disk);
endproc 
proc disk_unshow;peg,disk,level
rem
rem   disk_unshow
rem
rem    disk_unshow will remove a disk from a peg. Before a disk is put 
rem    on a new peg it must be removed from its old place on the screen.
rem
if peg=1: let peg=15
else : if peg=2: let peg=40
else : let peg=65: endif : endif 
print at 20-level,peg-disk;" ";rept(" ",2*disk);
print at 20-level,peg;"|";
endproc 
proc move;disk,p1,p2
rem
rem   move
rem
rem    move is the recursive procedure which will move a set of disks
rem    on peg p1 to peg p2. First the unused peg has to be figured out so
rem    the disk-1 disks can be put on it. This procedure is recursive unless
rem    disk=1 in which case the move can actually occur and the calls can
rem    begin working their way back. The variable p3 must be local or the
rem    next recursion will reset the value.
rem
local p3
if p1<>1 and p2<>1
let p3=1
else : if p1<>2 and p2<>2
let p3=2
else : let p3=3
endif : endif 
if disk=1
take;p1,1
put;p2,1
else 
move;disk-1,p1,p3
take;p1,disk
put;p2,disk
move;disk-1,p3,p2
endif 
endproc 
proc put;peg,disk
rem
rem   put
rem
rem    put will put the specified disk on next free level of the specified
rem    If there is no room on the peg the program stops. This will not occur
rem    the program was typed in with an error. The disk is show on the screen
rem    by the call to disk_show.
rem
first : while fieldv(peg-1) and not eof(): next : endwhile 
if eof()
print "No room on peg ";peg;" for disk ";disk
stop 
endif 
if peg=1: let peg1=disk
else : if peg=2: let peg2=disk
else : let peg3=disk: endif : endif 
update 
disk_show;peg,disk,recnum()+1
let moves=moves+1
endproc 
proc show_pegs
rem
rem   show_pegs
rem
rem    show_pegs sets up the initial screen to show all the disks on
rem    peg 1.
rem
print at 20,1;rept("-",78)
let i=1: first : while not eof()
disk_show;1,peg1,i
print tab 40;"|"; tab 65;"|"
let i=i+1: next : endwhile 
print at 20-i,15;"|"; tab 40;"|"; tab 65;"|"
endproc 
proc take;peg,disk
rem
rem   take
rem
rem    take will remove the specified disk from the specified peg. If it is
rem    not found then the program will stop. This will not occur unless there
rem    was an error when the program was typed in. The disk is remove from
rem    the screen by the call to disk_unshow.
rem
first : while fieldv(peg-1)<>disk and not eof(): next : endwhile 
if eof()
print "Error disk ";disk;" not found on peg ";peg
stop 
endif 
if peg=1
let peg1=0
else : if peg=2
let peg2=0
else : let peg3=0: endif : endif 
update 
disk_unshow;peg,disk,recnum()+1
endproc 
proc tower
rem
rem   tower
rem
rem    tower is the main procedure of the tower program. It sets the initial
rem    conditions and prompts for how many disks to use. It displays the
rem    pegs at the start and then starts the recursive calls to move.
rem
let td$="ram1_": let moves=0
mode 0
print " TOWER OF HANOI                                     B Cable   1/96"
print 
input " Start with how many disk [2-15] or 0 to stop? ";nd
let nd=int(nd)
if nd=0: mode 1: stop : endif 
if nd<2 or nd>15:tower: endif 
create_pegs;nd
show_pegs
move;nd-1,1,2
take;1,nd
put;3,nd
move;nd-1,2,3
close_pegs
print at 22,1;"Done after ";moves;" moves"; at 24,1;
endproc 
